@@ -49,11 +49,13 @@
  *					  and Luis Miguel Silveira, MIT RLE.
  */
 
+#include <config.h>
+
 #include <kdebug.h>
 #include <kmessagebox.h>
 #include <klocale.h>
 
-#include "dviwin.h"
+#include "dviRenderer.h"
 #include "xdvi.h"
 
 
