@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #ifndef MAINPAGE_H
@@ -77,7 +77,7 @@ namespace KMrmlConfig
 
         void slotMaybeIndex();
         void slotIndexingProgress( int percent, const QString& message );
-        void slotIndexingFinished( bool success );
+        void slotIndexingFinished( int returnCode );
         void slotCancelIndexing();
 
         
