@@ -1,6 +1,6 @@
 
 /*
-   Copyright (c) 2003-2004 Clarence Dang <dang@kde.org>
+   Copyright (c) 2003,2004,2005 Clarence Dang <dang@kde.org>
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -53,9 +53,9 @@
 #include <kptoolresizescale.h>
 #include <kptoolselection.h>
 #include <kptooltext.h>
-#include <kpview.h>
 #include <kpviewmanager.h>
 #include <kpviewscrollablecontainer.h>
+#include <kpzoomedview.h>
 
 
 // private
@@ -153,6 +153,13 @@ void kpMainWindow::enableEditMenuDocumentActions (bool enable)
 }
 
 
+// public
+QPopupMenu *kpMainWindow::selectionToolRMBMenu ()
+{
+    return (QPopupMenu *) guiFactory ()->container ("selectionToolRMBMenu", this);
+}
+
+
 // private slot
 void kpMainWindow::slotCut ()
 {
@@ -245,6 +252,14 @@ void kpMainWindow::slotEnablePaste ()
                         QTextDrag::canDecode (ms));
     #if DEBUG_KP_MAIN_WINDOW
         kdDebug () << "\t" << name () << "***canDecode=" << timer.restart () << endl;
+        for (int i = 0; ; i++)
+        {
+            const char *fmt = ms->format (i);
+            if (!fmt)
+                break;
+
+            kdDebug () << "\t'" << fmt << "'" << endl;
+        }
     #endif
     }
 
@@ -275,7 +290,7 @@ QRect kpMainWindow::calcUsefulPasteRect (int pixmapWidth, int pixmapHeight)
         const QPoint viewTopLeft (m_scrollView->contentsX (),
                                   m_scrollView->contentsY ());
 
-        const QPoint docTopLeft = m_mainView->zoomViewToDoc (viewTopLeft);
+        const QPoint docTopLeft = m_mainView->transformViewToDoc (viewTopLeft);
 
         if ((docTopLeft.x () + pixmapWidth <= m_document->width () &&
              docTopLeft.y () + pixmapHeight <= m_document->height ()) ||
@@ -368,6 +383,10 @@ void kpMainWindow::pasteText (const QString &text,
                << ")" << endl;
 #endif
 
+    if (text.isEmpty ())
+        return;
+
+
     // sync: restoreOverrideCursor() in all exit paths
     QApplication::setOverrideCursor (Qt::waitCursor);
 
@@ -385,12 +404,6 @@ void kpMainWindow::pasteText (const QString &text,
             textLines [textLines.size () - 1].append (text [i]);
     }
 
-    if (textLines.size () == 1 && textLines [0].isEmpty ())
-    {
-        QApplication::restoreOverrideCursor ();
-        return;
-    }
-
 
     if (!forceNewTextSelection &&
         m_document && m_document->selection () &&
