@@ -155,7 +155,7 @@ void KookaPreferences::setupOCRPage()
     /*
      *  Now read the config value CFG_OCR_ENGINE and set the radios to the value if available
      */
-    QString useEngine = konf->readEntry( CFG_OCR_ENGINE, "notFound" );
+    QString useEngine = konf->readEntry( CFG_OCR_ENGINE, "ocrad" );
     if( useEngine != "notFound" )
     {
         if( useEngine == "gocr" && haveGocr )
@@ -350,8 +350,8 @@ void KookaPreferences::setupStartupPage()
 void KookaPreferences::setupSaveFormatPage( )
 {
    konf->setGroup( OP_FILE_GROUP );
-   QFrame *page = addPage( i18n("Image Saving"), i18n("Configure the Image Save Assistant" ),
-			    BarIcon("hdd_unmount", KIcon::SizeMedium ) );
+   QFrame *page = addPage( i18n("Image Saving"), i18n("Configure Image Save Assistant" ),
+			    BarIcon("filesave", KIcon::SizeMedium ) );
    QVBoxLayout *top = new QVBoxLayout( page, 0, spacingHint() );
 
    /* Skip the format asking if a format entry  exists */
@@ -361,6 +361,14 @@ void KookaPreferences::setupSaveFormatPage( )
    QToolTip::add( cbSkipFormatAsk, i18n("Check this if you want to see the image save assistant even if there is a default format for the image type." ));
    top->addWidget( cbSkipFormatAsk );
 
+   cbFilenameAsk = new QCheckBox( i18n("Ask for filename when saving file"),
+                    page,  "CB_ASK_FILENAME" );
+   cbFilenameAsk->setChecked( konf->readBoolEntry( OP_ASK_FILENAME, false));
+   QToolTip::add( cbFilenameAsk, i18n("Check this if you want to enter a filename when an image has been scanned." ));
+   top->addWidget( cbFilenameAsk );
+
+
+
    top->addStretch(10);
 }
 
@@ -369,7 +377,7 @@ void KookaPreferences::setupThumbnailPage()
    konf->setGroup( THUMB_GROUP );
 
    QFrame *page = addPage( i18n("Thumbnail View"), i18n("Thumbnail Gallery View" ),
-			    BarIcon("appearance", KIcon::SizeMedium ) );
+			    BarIcon("thumbnail", KIcon::SizeMedium ) );
    QVBoxLayout *top = new QVBoxLayout( page, 0, spacingHint() );
 
    top->addWidget( new QLabel( i18n("Here you can configure the appearance of the thumbnail view of your scan picture gallery."),page ));
@@ -465,6 +473,7 @@ void KookaPreferences::slotApply( void )
     konf->setGroup( OP_FILE_GROUP );
     bool showFormatAssist = cbSkipFormatAsk->isChecked();
     konf->writeEntry( OP_FILE_ASK_FORMAT, showFormatAssist );
+    konf->writeEntry( OP_ASK_FILENAME, cbFilenameAsk->isChecked() );
 
     /* ** Thumbnail options ** */
     konf->setGroup( THUMB_GROUP );
@@ -485,7 +494,7 @@ void KookaPreferences::slotApply( void )
 
     if( m_ocradBut->isChecked() )
         eng = "ocrad";
-    
+
     if( m_kadmosBut && m_kadmosBut->isChecked() )
         eng = "kadmos";
 
@@ -501,11 +510,11 @@ void KookaPreferences::slotApply( void )
 
     QString path = m_urlReqGocr->url();
     if( ! path.isEmpty() )
-        konf->writeEntry( CFG_GOCR_BINARY, path );
+        konf->writePathEntry( CFG_GOCR_BINARY, path );
 
     path = m_urlReqOcrad->url();
     if( ! path.isEmpty() )
-        konf->writeEntry( CFG_OCRAD_BINARY, path );
+        konf->writePathEntry( CFG_OCRAD_BINARY, path );
 
     konf->sync();
 
