@@ -109,6 +109,7 @@ bool PDFGenerator::loadDocument( const QString & filePath, QValueVector<KPDFPage
     bool firstInput = true;
     bool triedWallet = false;
     KWallet::Wallet * wallet = 0;
+    int keep = 1;
     while ( !pdfdoc->isOk() && pdfdoc->getErrorCode() == errEncrypted )
     {
         QCString password;
@@ -144,7 +145,7 @@ bool PDFGenerator::loadDocument( const QString & filePath, QValueVector<KPDFPage
             firstInput = false;
 
             // if the user presses cancel, abort opening
-            if ( KPasswordDialog::getPassword( password, prompt ) != KPasswordDialog::Accepted )
+            if ( KPasswordDialog::getPassword( password, prompt, wallet ? &keep : 0 ) != KPasswordDialog::Accepted )
             break;
         }
 
@@ -154,8 +155,8 @@ bool PDFGenerator::loadDocument( const QString & filePath, QValueVector<KPDFPage
         pdfdoc = new PDFDoc( new GString( QFile::encodeName( filePath ) ), pwd2, pwd2 );
             delete pwd2;
 
-        // 3. if the password is correct, store it to the wallet
-        if ( pdfdoc->isOk() && wallet && /*safety check*/ wallet->isOpen() )
+        // 3. if the password is correct and the user chose to remember it, store it to the wallet
+        if ( pdfdoc->isOk() && wallet && /*safety check*/ wallet->isOpen() && keep > 0 )
         {
             QString goodPass = QString::fromLocal8Bit( password.data() );
             wallet->writePassword( filePath.section('/', -1, -1), goodPass );
@@ -314,6 +315,7 @@ void PDFGenerator::generatePixmap( PixmapRequest * request )
     // note: thread safety is set on 'false' for the GUI (this) thread
     kpdfOutputDev->setParams( request->width, request->height, genObjectRects, genObjectRects, false );
     pdfdoc->displayPage( kpdfOutputDev, page->number() + 1, fakeDpiX, fakeDpiY, 0, false, true, genObjectRects );
+    delete pdfdoc->takeLinks(); // we have already created the KPDFLinks so free the xpdf memory
 
     // 2. Take data from outputdev and attach it to the Page
     page->setPixmap( request->id, kpdfOutputDev->takePixmap() );
@@ -407,14 +409,21 @@ void PDFGenerator::putFontInfo(KListView *list)
 bool PDFGenerator::print( KPrinter& printer )
 {
     QString ps = printer.option("PageSize");
+    int paperWidth, paperHeight;
+    int marginTop, marginLeft, marginRight, marginBottom;
+    marginTop = (int)printer.option("kde-margin-top").toDouble();
+    marginLeft = (int)printer.option("kde-margin-left").toDouble();
+    marginRight = (int)printer.option("kde-margin-right").toDouble();
+    marginBottom = (int)printer.option("kde-margin-bottom").toDouble();
+
     if (ps.find(QRegExp("w\\d+h\\d+")) == 0)
     {
         // size not supported by Qt, KPrinter gives us the size as wWIDTHhHEIGHT
         // remove the w
         ps = ps.mid(1);
         int hPos = ps.find("h");
-        globalParams->setPSPaperWidth(ps.left(hPos).toInt());
-        globalParams->setPSPaperHeight(ps.mid(hPos+1).toInt());
+        paperWidth = ps.left(hPos).toInt();
+        paperHeight = ps.mid(hPos+1).toInt();
     }
     else
     {
@@ -424,17 +433,38 @@ bool PDFGenerator::print( KPrinter& printer )
         dummy.setPageSize((QPrinter::PageSize)(ps.isEmpty() ? KGlobal::locale()->pageSize() : pageNameToPageSize(ps)));
 
         QPaintDeviceMetrics metrics(&dummy);
-        globalParams->setPSPaperWidth(metrics.width());
-        globalParams->setPSPaperHeight(metrics.height());
+        paperWidth = metrics.width();
+        paperHeight = metrics.height();
     }
 
     KTempFile tf( QString::null, ".ps" );
-    PSOutputDev *psOut = new PSOutputDev(tf.name().latin1(), pdfdoc->getXRef(), pdfdoc->getCatalog(), 1, pdfdoc->getNumPages(), psModePS);
+    globalParams->setPSPaperWidth(paperWidth);
+    globalParams->setPSPaperHeight(paperHeight);
+    QString pstitle = getDocumentInfo("Title", true);
+    if ( pstitle.isEmpty() )
+    {
+        pstitle = m_document->currentDocument().fileName( false );
+    }
+    const char* pstitlechar = !pstitle.isEmpty() ? pstitle.local8Bit() : 0;
+    PSOutputDev *psOut = new PSOutputDev(tf.name().latin1(), pstitlechar, pdfdoc->getXRef(), pdfdoc->getCatalog(), 1, pdfdoc->getNumPages(), psModePS, marginRight, marginBottom, paperWidth - marginLeft, paperHeight - marginTop);
 
     if (psOut->isOk())
     {
         std::list<int> pages;
 
+        double xScale = ((double)paperWidth - (double)marginLeft - (double)marginRight) / (double)paperWidth;
+        double yScale = ((double)paperHeight - (double)marginBottom - (double)marginTop) / (double)paperHeight;
+
+        if ( abs((int)(xScale * 100) - (int)(yScale * 100)) > 5 ) {
+            int result = KMessageBox::questionYesNo(0,
+                                       i18n("The margins you specified change the page aspect ratio. Do you want to print with the aspect ratio changed or do you want the margins to be adapted so that the aspect ratio is preserved?"),
+                                       i18n("Aspect ratio change"),
+                                       i18n("Print with specified margins"),
+                                       i18n("Print adapting margins to keep aspect ratio"),
+                                       "kpdfStrictlyObeyMargins");
+            if (result == KMessageBox::Yes) psOut->setScale(xScale, yScale);
+        }
+
         if (!printer.previewOnly())
         {
             QValueList<int> pageList = printer.pageList();
@@ -662,17 +692,17 @@ void PDFGenerator::scanFont(GfxFont *font, KListView *list, Ref **fonts, int &fo
     (*fonts)[fontsLen++] = *font->getID();
 }
 
-QString PDFGenerator::getDocumentInfo( const QString & data ) const
+QString PDFGenerator::getDocumentInfo( const QString & data, bool canReturnNull ) const
 // note: MUTEX is LOCKED while calling this
 {
     // [Albert] Code adapted from pdfinfo.cc on xpdf
     Object info;
     if ( !pdfdoc )
-        return i18n( "Unknown" );
+        return canReturnNull ? QString::null : i18n( "Unknown" );
 
     pdfdoc->getDocInfo( &info );
     if ( !info.isDict() )
-        return i18n( "Unknown" );
+        return canReturnNull ? QString::null : i18n( "Unknown" );
 
     QString result;
     Object obj;
@@ -715,7 +745,7 @@ QString PDFGenerator::getDocumentInfo( const QString & data ) const
     }
     obj.free();
     info.free();
-    return i18n( "Unknown" );
+    return canReturnNull ? QString::null : i18n( "Unknown" );
 }
 
 QString PDFGenerator::getDocumentDate( const QString & data ) const
@@ -731,7 +761,7 @@ QString PDFGenerator::getDocumentDate( const QString & data ) const
         return i18n( "Unknown Date" );
 
     Object obj;
-    const char *s;
+    const char *s, *origS;
     int year, mon, day, hour, min, sec;
     Dict *infoDict = info.getDict();
     UnicodeMap *uMap = globalParams->getTextEncoding();
@@ -743,6 +773,7 @@ QString PDFGenerator::getDocumentDate( const QString & data ) const
     if ( infoDict->lookup( data.latin1(), &obj )->isString() )
     {
         s = UGString(*obj.getString()).getCString();
+        origS = s;
         if ( s[0] == 'D' && s[1] == ':' )
             s += 2;
 
@@ -757,6 +788,7 @@ QString PDFGenerator::getDocumentDate( const QString & data ) const
         }
         else
             result = s;
+        delete[] origS;
     }
     else
         result = i18n( "Unknown Date" );
@@ -831,7 +863,8 @@ void PDFGenerator::fillViewportFromLink( DocumentViewport &viewport, LinkDest *d
         viewport.pageNumber = pdfdoc->findPage( ref.num, ref.gen ) - 1;
     }
 
-    if (viewport.pageNumber == -1) return;
+    if (viewport.pageNumber < 0) return;
+    if (viewport.pageNumber >= pdfdoc->getNumPages()) return;
 
     // get destination position
     // TODO add other attributes to the viewport (taken from link)
@@ -1130,6 +1163,7 @@ void PDFPixmapGeneratorThread::run()
                                             genObjectRects, genObjectRects, TRUE /*thread safety*/ );
     d->generator->pdfdoc->displayPage( d->generator->kpdfOutputDev, page->number() + 1,
                                        fakeDpiX, fakeDpiY, 0, false, true, genObjectRects );
+    delete d->generator->pdfdoc->takeLinks(); // we have already created the KPDFLinks so free the xpdf memory
 
     // 2. grab data from the OutputDev and store it locally (note takeIMAGE)
 #ifndef NDEBUG
