@@ -526,6 +526,7 @@ void Part::slotDoFileDirty()
   if (m_viewportDirty.pageNumber == -1)
   {
     m_viewportDirty = m_document->viewport();
+    m_wasPresentationOpen = ((PresentationWidget*)m_presentationWidget != 0);
     m_pageView->showText(i18n("Reloading the document..."), 0);
   }
 
@@ -534,6 +535,7 @@ void Part::slotDoFileDirty()
     if (m_viewportDirty.pageNumber >= (int)m_document->pages()) m_viewportDirty.pageNumber = (int)m_document->pages() - 1;
     m_document->setViewport(m_viewportDirty);
     m_viewportDirty.pageNumber = -1;
+    if (m_wasPresentationOpen) slotShowPresentation();
     emit enablePrintAction(true);
   }
   else
@@ -789,7 +791,6 @@ void Part::slotPrintPreview()
 
     printer.setMinMax(1, m_document->pages());
     printer.setPreviewOnly( true );
-    printer.setMargins(0, 0, 0, 0);
 
     // if some pages are landscape and others are not the most common win as kprinter does
     // not accept a per page setting
@@ -921,7 +922,6 @@ void Part::slotPrint()
     printer.setPageSelection(KPrinter::ApplicationSide);
     printer.setMinMax(1, m_document->pages());
     printer.setCurrentPage(m_document->currentPage()+1);
-    printer.setMargins(0, 0, 0, 0);
 
     // if some pages are landscape and others are not the most common win as kprinter does
     // not accept a per page setting
@@ -955,15 +955,21 @@ void Part::doPrint(KPrinter &printer)
     }
 }
 
-void Part::restoreDocument(const KURL &url, int page)
+void Part::restoreDocument(KConfig* config)
 {
-  if (openURL(url)) goToPage(page);
+  KURL url ( config->readPathEntry( "URL" ) );
+  if ( url.isValid() )
+  {
+    QString viewport = config->readEntry( "Viewport" );
+    if (!viewport.isEmpty()) m_document->setNextDocumentViewport( DocumentViewport( viewport ) );
+    openURL( url );
+  }
 }
 
 void Part::saveDocumentRestoreInfo(KConfig* config)
 {
   config->writePathEntry( "URL", url().url() );
-  if (m_document->pages() > 0) config->writeEntry( "Page", m_document->currentPage() + 1 );
+  config->writeEntry( "Viewport", m_document->viewport().toString() );
 }
 
 /*
