@@ -47,6 +47,7 @@ Shell::Shell()
 }
 
 Shell::Shell(const KURL &url)
+ : KParts::MainWindow(0, "KPDF::Shell"), m_menuBarWasShown(true), m_toolBarWasShown(true)
 {
   m_openUrl = url;
   init();
@@ -86,7 +87,7 @@ void Shell::init()
     m_part = 0;
     return;
   }
-  connect( this, SIGNAL( restoreDocument(const KURL &, int) ),m_part, SLOT( restoreDocument(const KURL &, int)));
+  connect( this, SIGNAL( restoreDocument(KConfig*) ),m_part, SLOT( restoreDocument(KConfig*)));
   connect( this, SIGNAL( saveDocumentRestoreInfo(KConfig*) ), m_part, SLOT( saveDocumentRestoreInfo(KConfig*)));
   connect( m_part, SIGNAL( enablePrintAction(bool) ), m_printAction, SLOT( setEnabled(bool)));
   
@@ -116,8 +117,13 @@ void Shell::openURL( const KURL & url )
     if ( m_part )
     {
         bool openOk = m_part->openURL( url );
-        if ( openOk ) m_recent->addURL( url );
-        else m_recent->removeURL( url );
+        if ( openOk )
+        {
+          m_recent->addURL( url );
+          setCaption(url.fileName());
+        }
+        else
+          m_recent->removeURL( url );
     }
 }
 
@@ -174,8 +180,7 @@ void Shell::readProperties(KConfig* config)
   // in 'saveProperties'
   if(m_part)
   {
-    KURL url ( config->readPathEntry( "URL" ) );
-    if ( url.isValid() ) emit restoreDocument(url, config->readNumEntry( "Page", 1 ));
+    emit restoreDocument(config);
   }
 }
 
