@@ -97,6 +97,10 @@ public:
     bool blockPixmapsRequest;           // prevent pixmap requests
     PageViewMessage * messageWindow;    // in pageviewutils.h
 
+    // drag scroll
+    QPoint dragScrollVector;
+    QTimer dragScrollTimer;
+
     // actions
     KToggleAction * aMouseNormal;
     KToggleAction * aMouseSelect;
@@ -158,9 +162,7 @@ PageView::PageView( QWidget *parent, KPDFDocument *document )
 
     // conntect the padding of the viewport to pixmaps requests
     connect( this, SIGNAL(contentsMoving(int, int)), this, SLOT(slotRequestVisiblePixmaps(int, int)) );
-
-    // show initial welcome text
-    d->messageWindow->display( i18n( "Welcome" ), PageViewMessage::Info, 2000 );
+    connect( &d->dragScrollTimer, SIGNAL(timeout()), this, SLOT(slotDragScroll()) );
 
     // set a corner button to resize the view to the page size
 //    QPushButton * resizeButton = new QPushButton( viewport() );
@@ -168,10 +170,18 @@ PageView::PageView( QWidget *parent, KPDFDocument *document )
 //    setCornerWidget( resizeButton );
 //    resizeButton->setEnabled( false );
     // connect(...);
+    setInputMethodEnabled( true );
+
+    // schedule the welcome message
+    QTimer::singleShot( 0, this, SLOT( slotShowWelcome() ) );
 }
 
 PageView::~PageView()
 {
+    // delete all widgets
+    QValueVector< PageViewItem * >::iterator dIt = d->items.begin(), dEnd = d->items.end();
+    for ( ; dIt != dEnd; ++dIt )
+        delete *dIt;
     d->document->removeObserver( this );
     delete d;
 }
@@ -573,7 +583,7 @@ void PageView::keyPressEvent( QKeyEvent * e )
     e->accept();
 
     // if performing a selection or dyn zooming, disable keys handling
-    if ( !d->mouseSelectionRect.isNull() || d->mouseMidStartY != -1 )
+    if ( ( !d->mouseSelectionRect.isNull() && e->key() != Qt::Key_Escape ) || d->mouseMidStartY != -1 )
         return;
 
     // handle 'find as you type' (based on khtml/khtmlview.cpp)
@@ -625,12 +635,7 @@ void PageView::keyPressEvent( QKeyEvent * e )
         else if( !e->text().isEmpty() )
         {
             d->typeAheadString += e->text();
-            bool found = d->document->searchText( PAGEVIEW_SEARCH_ID, d->typeAheadString, false, false,
-                    KPDFDocument::NextMatch, true, qRgb( 128, 255, 128 ), true );
-            QString status = found ? i18n("Text found: \"%1\".") : i18n("Text not found: \"%1\".");
-            d->messageWindow->display( status.arg(d->typeAheadString.lower()),
-                                       found ? PageViewMessage::Find : PageViewMessage::Warning, 4000 );
-            d->findTimeoutTimer->start( 3000, true );
+            doTypeAheadSearch();
         }
         return;
     }
@@ -715,6 +720,15 @@ void PageView::keyPressEvent( QKeyEvent * e )
         case Key_Right:
             horizontalScrollBar()->addLine();
             break;
+        case Qt::Key_Escape:
+            selectionClear();
+            d->mousePressPos = QPoint();
+            if ( d->aPrevAction )
+            {
+                d->aPrevAction->activate();
+                d->aPrevAction = 0;
+            }
+            break;
         case Key_Shift:
         case Key_Control:
             if ( d->autoScrollTimer )
@@ -738,6 +752,19 @@ void PageView::keyPressEvent( QKeyEvent * e )
     }
 }
 
+void PageView::imEndEvent( QIMEvent * e )
+{
+    if( d->typeAheadActive )
+    {
+        if( !e->text().isEmpty() )
+        {
+            d->typeAheadString += e->text();
+            doTypeAheadSearch();
+            e->accept();
+        }
+    }
+}
+
 void PageView::contentsMouseMoveEvent( QMouseEvent * e )
 {
     // don't perform any mouse action when no document is shown
@@ -749,7 +776,7 @@ void PageView::contentsMouseMoveEvent( QMouseEvent * e )
         return;
 
     // if holding mouse mid button, perform zoom
-    if ( (e->state() & MidButton) && d->mouseMidStartY > 0 )
+    if ( (e->state() & MidButton) && d->mouseMidStartY >= 0 )
     {
         int deltaY = d->mouseMidStartY - e->globalPos().y();
         d->mouseMidStartY = e->globalPos().y();
@@ -801,7 +828,7 @@ void PageView::contentsMouseMoveEvent( QMouseEvent * e )
         case MouseZoom:
         case MouseSelect:
             // set second corner of selection
-            if ( leftButton || d->aPrevAction )
+            if ( !d->mousePressPos.isNull() && ( leftButton || d->aPrevAction ) )
                 selectionEndPoint( e->x(), e->y() );
             break;
 
@@ -875,6 +902,9 @@ void PageView::contentsMousePressEvent( QMouseEvent * e )
 
 void PageView::contentsMouseReleaseEvent( QMouseEvent * e )
 {
+    // stop the drag scrolling
+    d->dragScrollTimer.stop();
+
     // don't perform any mouse action when no document is shown
     if ( d->items.isEmpty() )
     {
@@ -1152,6 +1182,12 @@ void PageView::wheelEvent( QWheelEvent *e )
     if ( d->viewportMoveActive )
         return;
 
+    if ( !d->document->isOpened() )
+    {
+        QScrollView::wheelEvent( e );
+        return;
+    }
+
     int delta = e->delta(),
         vScroll = verticalScrollBar()->value();
     e->accept();
@@ -1349,6 +1385,20 @@ void PageView::selectionStart( int x, int y, const QColor & color, bool /*aboveA
 
 void PageView::selectionEndPoint( int x, int y )
 {
+    if (x < contentsX()) d->dragScrollVector.setX(x - contentsX());
+    else if (contentsX() + viewport()->width() < x) d->dragScrollVector.setX(x - contentsX() - viewport()->width());
+    else d->dragScrollVector.setX(0);
+
+    if (y < contentsY()) d->dragScrollVector.setY(y - contentsY());
+    else if (contentsY() + viewport()->height() < y) d->dragScrollVector.setY(y - contentsY() - viewport()->height());
+    else d->dragScrollVector.setY(0);
+
+    if (d->dragScrollVector != QPoint(0, 0))
+    {
+       if (!d->dragScrollTimer.isActive()) d->dragScrollTimer.start(100);
+    }
+    else d->dragScrollTimer.stop();
+
     // clip selection to the viewport
     QRect viewportRect( contentsX(), contentsY(), visibleWidth(), visibleHeight() );
     x = QMAX( QMIN( x, viewportRect.right() ), viewportRect.left() );
@@ -1524,6 +1574,16 @@ void PageView::updateCursor( const QPoint &p )
     }
 }
 
+void PageView::doTypeAheadSearch()
+{
+    bool found = d->document->searchText( PAGEVIEW_SEARCH_ID, d->typeAheadString, false, false,
+                                          KPDFDocument::NextMatch, true, qRgb( 128, 255, 128 ), true );
+    QString status = found ? i18n("Text found: \"%1\".") : i18n("Text not found: \"%1\".");
+    d->messageWindow->display( status.arg(d->typeAheadString.lower()),
+                               found ? PageViewMessage::Find : PageViewMessage::Warning, 4000 );
+    d->findTimeoutTimer->start( 3000, true );
+}
+
 //BEGIN private SLOTS
 void PageView::slotRelayoutPages()
 // called by: notifySetup, viewportResizeEvent, slotTwoPagesToggled, slotContinuousToggled, updateZoom
@@ -1782,16 +1842,8 @@ void PageView::slotRequestVisiblePixmaps( int newLeft, int newTop )
          KpdfSettings::memoryLevel() != KpdfSettings::EnumMemoryLevel::Low &&
          KpdfSettings::enableThreading() )
     {
-        // add the page before the 'visible series' in preload
-        int headRequest = d->visibleItems.first()->pageNumber() - 1;
-        if ( headRequest >= 0 )
-        {
-            PageViewItem * i = d->items[ headRequest ];
-            // request the pixmap if not already present
-            if ( !i->page()->hasPixmap( PAGEVIEW_ID, i->width(), i->height() ) && i->width() > 0 )
-                requestedPixmaps.push_back( new PixmapRequest(
-                        PAGEVIEW_ID, i->pageNumber(), i->width(), i->height(), PAGEVIEW_PRELOAD_PRIO, true ) );
-        }
+        // as the requests are done in the order as they appear in the list,
+        // request first the next page and then the previous
 
         // add the page after the 'visible series' in preload
         int tailRequest = d->visibleItems.last()->pageNumber() + 1;
@@ -1803,6 +1855,17 @@ void PageView::slotRequestVisiblePixmaps( int newLeft, int newTop )
                 requestedPixmaps.push_back( new PixmapRequest(
                         PAGEVIEW_ID, i->pageNumber(), i->width(), i->height(), PAGEVIEW_PRELOAD_PRIO, true ) );
         }
+
+        // add the page before the 'visible series' in preload
+        int headRequest = d->visibleItems.first()->pageNumber() - 1;
+        if ( headRequest >= 0 )
+        {
+            PageViewItem * i = d->items[ headRequest ];
+            // request the pixmap if not already present
+            if ( !i->page()->hasPixmap( PAGEVIEW_ID, i->width(), i->height() ) && i->width() > 0 )
+                requestedPixmaps.push_back( new PixmapRequest(
+                        PAGEVIEW_ID, i->pageNumber(), i->width(), i->height(), PAGEVIEW_PRELOAD_PRIO, true ) );
+        }
     }
 
     // send requests to the document
@@ -1872,6 +1935,13 @@ void PageView::slotAutoScoll()
     scrollBy( 0, d->scrollIncrement > 0 ? scrollOffset[ index ] : -scrollOffset[ index ] );
 }
 
+void PageView::slotDragScroll()
+{
+    scrollBy(d->dragScrollVector.x(), d->dragScrollVector.y());
+    QPoint p = viewportToContents( mapFromGlobal( QCursor::pos() ) );
+    selectionEndPoint( p.x(), p.y() );
+}
+
 void PageView::findAheadStop()
 {
     d->typeAheadActive = false;
@@ -1883,6 +1953,12 @@ void PageView::findAheadStop()
     releaseKeyboard();
 }
 
+void PageView::slotShowWelcome()
+{
+    // show initial welcome text
+    d->messageWindow->display( i18n( "Welcome" ), PageViewMessage::Info, 2000 );
+}
+
 void PageView::slotZoom()
 {
     setFocus();
