@@ -22,7 +22,7 @@
 #include <config.h>
 #endif
 
-#include "pmnamedobjectedit.h"
+#include "pmdetailobjectedit.h"
 
 class PMBlobSphere;
 class PMVectorEdit;
@@ -31,10 +31,10 @@ class PMFloatEdit ;
 /**
  * Dialog edit class for @ref PMBlobSphere
  */
-class PMBlobSphereEdit : public PMNamedObjectEdit
+class PMBlobSphereEdit : public PMDetailObjectEdit
 {
    Q_OBJECT
-   typedef PMNamedObjectEdit Base;
+   typedef PMDetailObjectEdit Base;
 public:
    /**
     * Creates a PMBlobSphereEdit with parent and name
