@@ -51,7 +51,7 @@ PMLibraryObjectSearch::PMLibraryObjectSearch( QWidget* parent ) :
    grid->addWidget( m_pSearch, 0, 1 );
 
    QVBoxLayout* fvl = new QVBoxLayout( fhl );
-   m_pSearchButton = new QPushButton( i18n( "Search" ), frame );
+   m_pSearchButton = new QPushButton( i18n( "&Search" ), frame );
    fvl->addWidget( m_pSearchButton );
    fvl->addStretch( 1 );
 
